from finvizfinance.group.overview import Overview
"""
.. module:: group.valuation
   :synopsis: group valuation table.

.. moduleauthor:: Tianning Li <ltianningli@gmail.com>
"""

class Valuation(Overview):
    """Valuation inherit from overview module.
    Getting information from the finviz group valuation page.
    """
    def __init__(self):
        """initiate module
        """
        self.BASE_URL = 'https://finviz.com/groups.ashx?{group}&v=120'
        self.url = self.BASE_URL.format(group='g=sector')
        Overview._loadSetting(self)


from finvizfinance.group.overview import Overview
from finvizfinance.util import webScrap, imageScrap

"""
.. module:: group.spectrum
   :synopsis: group spectrum image.

.. moduleauthor:: Tianning Li <ltianningli@gmail.com>
"""

class Spectrum(Overview):
    """Spectrum inherit from overview module.
    Getting information from the finviz group spectrum page.
    """
    def __init__(self):
        """initiate module
        """
        self.BASE_URL = 'https://finviz.com/groups.ashx?{group}&v=310'
        self.url = self.BASE_URL.format(group='g=sector')
        Overview._loadSetting(self)

    def ScreenerView(self, group='Sector', order='Name', out_dir=''):
        """Get screener table.

        Args:
            group(str): choice of group option.
            order(str): sort the table by the choice of order.
        """
        if group not in self.group_dict:
            raise ValueError()
        if order not in self.order_dict:
            raise ValueError()
        self.url = self.BASE_URL.format(group=self.group_dict[group])+'&'+self.order_dict[order]

        soup = webScrap(self.url)
        url = 'https://finviz.com/' + soup.findAll('img')[5]['src']
        imageScrap(url, group, '')


import os
import pandas as pd
from finvizfinance.screener.financial import Financial
from finvizfinance.screener.overview import Overview
from finvizfinance.screener.valuation import Valuation
from finvizfinance.screener.ownership import Ownership
from finvizfinance.screener.performance import Performance
from finvizfinance.screener.technical import Technical
"""
.. module:: earnings
   :synopsis: earnings.

.. moduleauthor:: Tianning Li <ltianningli@gmail.com>
"""

class Earnings:
    """Earnings
    Partition dataframe of ticker information of period of earning dates(This Week,
    Next Week, Previous Week, This Month) into dates
        
    Args:
        period(str): choose an option of period(This Week, Next Week,
                     Previous Week, This Month).
    """
    def __init__(self, period='This Week'):
        """initiate module
        """
        self.earning_days = []
        self.df_days = {}
        self.df = None
        self.period = period
        self._setPeriod(period)

    def _setPeriod(self, period):
        """Set the period.

        Args:
            period(str): choose an option of period(This Week, Next Week,
                         Previous Week, This Month).
        """
        check_list = ['This Week', 'Next Week', 'Previous Week','This Month']
        if period not in check_list:
            print('Available period: {}'.format(check_list))
            raise ValueError()
        self.period = period
        ffinancial = Financial()
        filters_dict = {'Earnings Date': period}
        ffinancial.set_filter(filters_dict=filters_dict)
        self.df = ffinancial.ScreenerView(order='Earnings Date', verbose=0)
        self.earning_days = list(set(self.df['Earnings'].to_list()))
        self.earning_days.sort()

    def partitionDays(self, mode='financial'):
        """Partition dataframe to separate dataframes according to the dates.

        Args:
            mode(str): choose an option of period(financial, overview, valuation, ownership,
                       performance, technical).
        """
        check_list = ['financial', 'overview', 'valuation', 'ownership','performance','technical']
        if mode not in check_list:
            print('Available mode: {}'.format(check_list))
            raise ValueError()

        for earning_day in self.earning_days:
            if mode == 'financial':
                self.df_days[earning_day] = self.df[self.df['Earnings'] == earning_day].reset_index(drop=True)
            else:
                self.df_days[earning_day] = self.df[self.df['Earnings'] == earning_day]['Ticker'].to_list()

        fearnings = None
        if mode == 'financial':
            return self.df_days
        elif mode == 'overview':
            fearnings = Overview()
        elif mode == 'valuation':
            fearnings = Valuation()
        elif mode == 'ownership':
            fearnings = Ownership()
        elif mode == 'performance':
            fearnings = Performance()
        elif mode == 'technical':
            fearnings = Technical()

        filters_dict = {'Earnings Date': self.period}
        fearnings.set_filter(filters_dict=filters_dict)
        df2 = fearnings.ScreenerView(order='Earnings Date', verbose=0)
        df2_days = {}
        for earning_day in self.earning_days:
            tickers = self.df_days[earning_day]
            df2_days[earning_day] = df2[df2['Ticker'].isin(tickers)].reset_index(drop=True)
        self.df_days = df2_days
        return self.df_days

    def outputExcel(self, output_file='earning_days.xlsx'):
        """Output dataframes to single Excel file.

        Args:
            output_file(str): name of the output excel file.
        """
        print('Print to Excel...')
        with pd.ExcelWriter(output_file,
                            datetime_format='YYYY-MM-DD',
                            engine='xlsxwriter') as writer:
            for name, df in self.df_days.items():
                sheet_name = '_'.join(name.split('/'))
                df.to_excel(writer, sheet_name=sheet_name, index=False)

    def outputCSV(self, output_dir='earning_days'):
        """Output dataframes to csv files.

        Args:
            output_dir(str): name of the output directory.
        """
        print('Print to CSV...')
        isdir = os.path.isdir(output_dir)
        if not isdir:
            os.mkdir(output_dir)
        for name, df in self.df_days.items():
            file_name = '_'.join(name.split('/'))
            df.to_csv(output_dir+'/'+file_name+'.csv',index=False)



from finvizfinance.util import webScrap, imageScrapFunction
import pandas as pd
import json

"""
.. module:: future
   :synopsis: future.

.. moduleauthor:: Tianning Li <ltianningli@gmail.com>
"""

class Future:
    """Future
    Getting information from the finviz future page.
    """
    def __init__(self):
        """initiate module
        """
        pass

    def performance(self, timeframe='D'):
        """Get forex performance table.

        Args:
            timeframe (str): choice of timeframe(D, W, M, Q, HY, Y)

        Returns:
            df(pandas.DataFrame): forex performance table
        """
        params = None
        if timeframe == 'D':
            params = ''
        elif timeframe == 'W':
            params = '?v=12'
        elif timeframe == 'M':
            params = '?v=13'
        elif timeframe == 'Q':
            params = '?v=14'
        elif timeframe == 'HY':
            params = '?v=15'
        elif timeframe == 'Y':
            params = '?v=16'
        else:
            raise ValueError()

        soup = webScrap('https://finviz.com/futures_performance.ashx'+params)
        data = soup.text[soup.text.find('var rows = ')+11:soup.text.find('FinvizInitFuturesPerformance(rows);')]
        data = json.loads(data.strip()[:-1])
        df = pd.DataFrame(data)
        return df



from finvizfinance.screener.overview import Overview
from finvizfinance.util import webScrap, progressBar
"""
.. module:: screen.ticker
   :synopsis: screen ticker table.

.. moduleauthor:: Tianning Li <ltianningli@gmail.com>
"""

class Ticker(Overview):
    """Financial inherit from overview module.
    Getting information from the finviz screener ticker page.
    """
    def __init__(self):
        """initiate module
        """
        self.BASE_URL = 'https://finviz.com/screener.ashx?v=411{signal}{filter}&ft=4{ticker}'
        self.url = self.BASE_URL.format(signal='', filter='',ticker='')
        Overview._loadSetting(self)

    def _screener_helper(self, i, page, soup, tickers, limit):
        table = soup.findAll('table')[18]
        page_tickers = table.findAll('span')
        if i == page - 1:
            page_tickers = page_tickers[:((limit - 1) % 1000 + 1)]
        tickers = tickers + [i.text.split('\xa0')[1] for i in page_tickers]
        return tickers

    def ScreenerView(self, limit=-1, verbose=1):
        """Get screener table.

        Args:
            verbose(int): choice of visual the progress. 1 for visualize progress.
        Returns:
            tickers(list): get all the tickers as list.
        """
        soup = webScrap(self.url)
        page = self._get_page(soup)
        if page == 0:
            if verbose == 1:
                print('No ticker found.')
            return None

        if limit != -1:
            if page > (limit-1)//1000+1:
                page = (limit-1)//1000+1

        if verbose == 1:
            progressBar(1, page)

        tickers = []
        tickers = self._screener_helper(0, page, soup, tickers, limit)

        for i in range(1, page):
            if verbose == 1:
                progressBar(i+1, page)
            soup = webScrap(self.url + '&r={}'.format(i * 1000 + 1))
            tickers = self._screener_helper(i, page, soup, tickers, limit)
        return tickers


from finvizfinance.group.overview import Overview
"""
.. module:: group.performance
   :synopsis: group performance table.

.. moduleauthor:: Tianning Li <ltianningli@gmail.com>
"""

class Performance(Overview):
    """Performance inherit from overview module.
    Getting information from the finviz group performance page.
    """
    def __init__(self):
        """initiate module
        """
        self.BASE_URL = 'https://finviz.com/groups.ashx?{group}&v=140'
        self.url = self.BASE_URL.format(group='g=sector')
        Overview._loadSetting(self)


from finvizfinance.util import webScrap, numberCovert
import pandas as pd
"""
.. module:: group.overview
   :synopsis: group overview table.

.. moduleauthor:: Tianning Li <ltianningli@gmail.com>

"""

class Overview:
    """Overview
    Getting information from the finviz group overview page.
    """
    def __init__(self):
        """initiate module"""
        self.BASE_URL = 'https://finviz.com/groups.ashx?{group}&v=110'
        self.url = self.BASE_URL.format(group='g=sector')
        self._loadSetting()

    def _loadSetting(self):
        """load all the groups."""
        soup = webScrap(self.url)
        selects = soup.findAll('select')

        # group
        options = selects[0].findAll('option')
        key = [i.text for i in options]
        value = []
        for option in options:
            temp = option['value'].split('?')[1].split('&')
            if len(temp) == 4:
                temp = '&'.join(temp[:2])
            else:
                temp = temp[0]
            value.append(temp)
        self.group_dict = dict(zip(key, value))

        # order
        options = selects[1].findAll('option')
        key = [i.text for i in options]
        value = [i['value'].split('&')[-1] for i in options]
        self.order_dict = dict(zip(key, value))

    def getGroup(self):
        """Get groups.

        Returns:
            groups(list): all the available groups.
        """
        return list(self.group_dict.keys())


    def getOrders(self):
        """Get orders.

        Returns:
            orders(list): all the available orders.
        """
        return list(self.order_dict.keys())

    def ScreenerView(self, group='Sector', order='Name'):
        """Get screener table.

        Args:
            group(str): choice of group option.
            order(str): sort the table by the choice of order.

        Returns:
            df(pandas.DataFrame): group information table.
        """
        if group not in self.group_dict:
            raise ValueError()
        if order not in self.order_dict:
            raise ValueError()
        self.url = self.BASE_URL.format(group=self.group_dict[group])+'&'+self.order_dict[order]

        soup = webScrap(self.url)
        table = soup.findAll('table')[5]
        rows = table.findAll('tr')
        table_header = [i.text for i in rows[0].findAll('td')][1:]
        df = pd.DataFrame([], columns=table_header)
        rows = rows[1:]
        num_col_index = [i for i in range(2, len(table_header))]
        for row in rows:
            cols = row.findAll('td')[1:]
            info_dict = {}
            for i, col in enumerate(cols):
                # check if the col is number
                if i not in num_col_index:
                    info_dict[table_header[i]] = col.text
                else:
                    info_dict[table_header[i]] = numberCovert(col.text)

            df = df.append(info_dict, ignore_index=True)
        return df



from finvizfinance.util import webScrap, imageScrap, numberCovert
import pandas as pd
from datetime import datetime
"""
.. module:: finvizfinance
   :synopsis: individual ticker.

.. moduleauthor:: Tianning Li <ltianningli@gmail.com>
"""
QUOTE_URL = 'https://finviz.com/quote.ashx?t={ticker}'

class finvizfinance:
    """finvizfinance
    Getting information from the individual ticker.

    Args:
        ticker(str): ticker string
        verbose(int): choice of visual the progress. 1 for visualize progress.
    """
    def __init__(self, ticker, verbose=0):
        """initiate module
        """
        self.ticker = ticker
        self.flag = False
        self.quote_url = QUOTE_URL.format(ticker=ticker)
        self.soup = webScrap(self.quote_url)
        if self._checkexist(verbose):
            self.flag = True
        self.info = {}

    def _checkexist(self, verbose):
        try:
            if 'not found' in self.soup.find('td', class_='body-text').text:
                print('Ticker not found.')
                return False
        except:
            if verbose == 1:
                print('Ticker exists.')
            return True

    def TickerCharts(self, timeframe='daily', charttype='advanced', out_dir='', urlonly=False):
        """Download ticker charts.

        Args:
            timeframe(str): choice of timeframe (daily, weekly, monthly).
            charttype(str): choice of type of chart (candle, line, advanced).
            out_dir(str): output image directory. default none.
            urlonly (bool): choice of downloading charts, default: downloading chart

        Returns:
            charturl(str): url for the chart
        """
        if timeframe not in ['daily','weekly','monthly']:
            raise ValueError()
        if charttype not in ['candle', 'line','advanced']:
            raise ValueError()
        url_type = 'c'
        url_ta = '0'
        if charttype == 'line':
            url_type = 'l'
        elif charttype == 'advanced' and timeframe != 'weekly' and timeframe != 'monthly':
            url_ta = '1'

        url_timeframe = 'd'
        if timeframe == 'week':
            url_timeframe = 'w'
        elif timeframe == 'monthly':
            url_timeframe = 'm'
        chart_url = 'https://finviz.com/chart.ashx?t={ticker}&ty={type}&ta={ta}&p={timeframe}'.format(ticker=self.ticker,
                                                    type=url_type, ta=url_ta, timeframe=url_timeframe)
        if not urlonly:
            imageScrap(chart_url, self.ticker, out_dir)
        return chart_url

    def TickerFundament(self):
        """Get ticker fundament.

        Returns:
            fundament(dict): ticker fundament.
        """
        fundament_info = {}

        table = self.soup.findAll('table')[5]
        rows = table.findAll('tr')

        fundament_info['Company'] = rows[1].text
        fundament_info['Sector'], fundament_info['Industry'], fundament_info['Country'] = rows[2].text.split(' | ')

        fundament_table = self.soup.find('table', class_='snapshot-table2')
        rows = fundament_table.findAll('tr')

        for row in rows:
            cols = row.findAll('td')
            cols = [i.text for i in cols]
            header = ''
            for i, value in enumerate(cols):
                if i % 2 == 0:
                    header = value
                else:
                    fundament_info[header] = value
        self.info['fundament'] = fundament_info
        return fundament_info

    def TickerDescription(self):
        """Get ticker description.

        Returns:
            description(str): ticker description.
        """
        return self.soup.find('td',class_='fullview-profile').text

    def TickerOuterRatings(self):
        """Get outer ratings table.

        Returns:
            df(pandas.DataFrame): outer ratings table
        """
        fullview_ratings_outer = self.soup.find('table', class_="fullview-ratings-outer")
        df = pd.DataFrame([], columns=['Date', 'Status', 'Outer', 'Rating', 'Price'])
        rows = fullview_ratings_outer.findAll('td', class_="fullview-ratings-inner")
        for row in rows:
            each_row = row.find('tr')
            cols = each_row.findAll('td')
            date = cols[0].text
            date = datetime.strptime(date, '%b-%d-%y')
            status = cols[1].text
            outer = cols[2].text
            rating = cols[3].text
            price = cols[4].text
            df = df.append({'Date': date, 'Status': status, 'Outer': outer, 'Rating': rating, 'Price': price},
                           ignore_index=True)
        self.info['ratings_outer'] = df
        return df

    def TickerNews(self):
        """Get news information table.

        Returns:
            df(pandas.DataFrame): news information table
        """
        fullview_news_outer = self.soup.find('table', class_='fullview-news-outer')
        rows = fullview_news_outer.findAll('tr')

        last_date = ''
        df = pd.DataFrame([], columns=['Date', 'Title', 'Link'])
        for row in rows:
            cols = row.findAll('td')
            date = cols[0].text
            title = cols[1].a.text
            link = cols[1].a['href']
            newsTime = date.split()
            if len(newsTime) == 2:
                last_date = newsTime[0]
                newsTime = ' '.join(newsTime)
            else:
                newsTime = last_date + ' ' + newsTime[0]
            newsTime = datetime.strptime(newsTime, '%b-%d-%y %I:%M%p')
            df = df.append({'Date': newsTime, 'Title': title, 'Link': link}, ignore_index=True)
        self.info['news'] = df
        return df

    def TickerInsideTrader(self):
        """Get insider information table.

        Returns:
            df(pandas.DataFrame): insider information table
        """
        inside_trader = self.soup.find('table', class_='body-table')
        rows = inside_trader.findAll('tr')
        table_header = [i.text for i in rows[0].findAll('td')]
        table_header += ['Insider_id']
        df = pd.DataFrame([], columns=table_header)
        rows = rows[1:]
        num_col = ['Cost','#Shares','Value ($)','#Shares Total']
        num_col_index = [table_header.index(i) for i in table_header if i in num_col]
        for row in rows:
            cols = row.findAll('td')
            info_dict = {}
            for i, col in enumerate(cols):
                if i not in num_col_index:
                    info_dict[table_header[i]] = col.text
                else:
                    info_dict[table_header[i]] = numberCovert(col.text)
            info_dict['Insider_id'] = cols[0].a['href'].split('oc=')[1].split('&tc=')[0]
            df = df.append(info_dict, ignore_index=True)
        self.info['inside trader'] = df
        return df

    def TickerSignal(self):
        """Get all the trading signals from finviz.

        Returns:
            ticker_signals(list): get all the ticker signals as list.
        """
        from finvizfinance.screener.ticker import Ticker
        fticker = Ticker()
        signals = ['Top Gainers', 'Top Losers', 'New High', 'New Low',
                   'Most Volatile', 'Most Active', 'Unusual Volume',
                   'Overbought', 'Oversold', 'Downgrades', 'Upgrades',
                   'Earnings Before', 'Earnings After', 'Recent Insider Buying',
                   'Recent Insider Selling', 'Major News', 'Horizontal S/R',
                   'TL Resistance', 'TL Support', 'Wedge Up', 'Wedge Down',
                   'Triangle Ascending', 'Triangle Descending', 'Wedge',
                   'Channel Up', 'Channel Down', 'Channel', 'Double Top',
                   'Double Bottom', 'Multiple Top', 'Multiple Bottom',
                   'Head & Shoulders', 'Head & Shoulders Inverse']
        ticker_signal = []
        for signal in signals:
            fticker.set_filter(signal=signal, ticker=self.ticker.upper())
            if fticker.ScreenerView(verbose=0) == [self.ticker.upper()]:
                ticker_signal.append(signal)
        return ticker_signal

    def TickerFullInfo(self):
        """Get all the ticker information.

        Returns:
            df(pandas.DataFrame): insider information table
        """
        self.TickerFundament()
        self.TickerOuterRatings()
        self.TickerNews()
        self.TickerInsideTrader()
        return self.info